from os import getcwd
from pathlib import Path


def create_project():
    Path(getcwd() + "/.ite").mkdir(parents=True, exist_ok=True)


def check_project_is_exist() -> bool:
    return Path(getcwd() + "/.ite").is_dir()


def check_project_content_is_exist() -> bool:
    return Path(getcwd() + "/.ite/ite_content.json").is_file()



import json
import os
from pathlib import Path
from threading import Lock

from integration_testing_environment.utils.content.ite_content_data import ite_content_data_dict
from integration_testing_environment.utils.exception.exceptions import ITEContentFileException
from integration_testing_environment.utils.json_format.json_process import reformat_json
from integration_testing_environment.utils.project.create_project import check_project_is_exist, check_project_content_is_exist
from integration_testing_environment.utils.project.create_project import create_project


def read_output_content():
    """
    read the editor content
    """
    lock = Lock()
    try:
        lock.acquire()
        cwd = os.getcwd()
        if check_project_is_exist() and check_project_content_is_exist():
            file_path = Path(cwd + "/.ite/ite_content.json")
            if file_path.exists() and file_path.is_file():
                with open(cwd + "/.ite/ite_content.json", "r+") as read_file:
                    return read_file.read()
    except ITEContentFileException:
        raise ITEContentFileException
    finally:
        lock.release()


def write_output_content():
    """
    write the editor content
    """
    lock = Lock()
    try:
        lock.acquire()
        cwd = os.getcwd()
        if check_project_is_exist():
            with open(cwd + "/.ite/ite_content.json", "w+") as file_to_write:
                file_to_write.write(reformat_json(json.dumps(ite_content_data_dict)))
        else:
            create_project()
            with open(cwd + "/.ite/ite_content.json", "w+") as file_to_write:
                file_to_write.write(reformat_json(json.dumps(ite_content_data_dict)))
    except ITEContentFileException:
        raise ITEContentFileException
    finally:
        lock.release()


def save_content_and_quit():
    """
    set content data and write
    """
    write_output_content()


def open_content_and_start():
    """
    read data and set content
    """
    temp_content = read_output_content()
    if temp_content is not None:
        return json.loads(temp_content)



from integration_testing_environment.utils.manager.package_manager.package_manager_class import package_manager


class HTMLReportManager(object):

    def __init__(self):
        self.html_report_manager_dict = {
            "je_auto_control": None,
            "je_api_testka": None,
            "je_load_density": None,
            "je_web_runner": None,
        }
        for package in package_manager.installed_package_dict.keys():
            if package in self.html_report_manager_dict.keys() and package_manager.installed_package_dict.get(
                    package) is not None:
                self.html_report_manager_dict.update(
                    {
                        package: package_manager.installed_package_dict.get(package).generate_html
                    }
                )


html_report_manager = HTMLReportManager()



import json
import sys

from integration_testing_environment.extend.mail_thunder_extend.mail_thunder_setting import send_after_test
from integration_testing_environment.utils.exception.exception_tag import wrong_test_data_format_exception_tag
from integration_testing_environment.utils.exception.exceptions import ITETestExecutorException
from integration_testing_environment.utils.test_executor.task_process_manager import TaskProcessManager


def call_auto_control_test(test_format_code: str, program_buffer: int = 1024000):
    try:
        TaskProcessManager(
            "je_auto_control",
            program_buffer_size=program_buffer
        ).start_test_process(
            "je_auto_control",
            exec_str=test_format_code,
        )
    except json.decoder.JSONDecodeError as error:
        print(
            repr(error) +
            "\n"
            + wrong_test_data_format_exception_tag,
            file=sys.stderr
        )
    except ITETestExecutorException as error:
        print(repr(error), file=sys.stderr)


def call_auto_control_test_with_send(test_format_code: str, program_buffer: int = 1024000):
    try:
        TaskProcessManager(
            title_name="je_auto_control",
            task_done_trigger_function=send_after_test,
            program_buffer_size=program_buffer
        ).start_test_process(
            "je_auto_control",
            exec_str=test_format_code,
        )
    except json.decoder.JSONDecodeError as error:
        print(
            repr(error) +
            "\n"
            + wrong_test_data_format_exception_tag,
            file=sys.stderr
        )
    except ITETestExecutorException as error:
        print(repr(error), file=sys.stderr)



from tkinter import NORMAL, END, DISABLED

from integration_testing_environment.utils.manager.redirect_manager.redirect_manager_class import \
    redirect_manager_instance


def redirect_output(ite_instance):
    ite_instance.program_run_result_textarea.configure(state=NORMAL)
    if not redirect_manager_instance.std_out_queue.empty():
        ite_instance.program_run_result_textarea.insert(
            END,
            redirect_manager_instance.std_out_queue.get_nowait(),
        )
    if not redirect_manager_instance.std_err_queue.empty():
        ite_instance.program_run_result_textarea.tag_configure("warning", foreground="red")
        ite_instance.program_run_result_textarea.insert(
            END,
            redirect_manager_instance.std_err_queue.get_nowait(),
            "warning",
        )
    ite_instance.program_run_result_textarea.configure(state=DISABLED)
    ite_instance.program_run_result_textarea.after(10, lambda: redirect_output(ite_instance))



import sys

from integration_testing_environment.utils.content.content_save import open_content_and_start
from integration_testing_environment.utils.exception.exceptions import ITEContentFileException


def content_init(ite_instance):
    ite_instance.file_from_output_content = open_content_and_start()
    try:
        if ite_instance.file_from_output_content is not None:
            if ite_instance.file_from_output_content.get("program_buffer") is not None:
                ite_instance.program_buffer = \
                    int(ite_instance.file_from_output_content.get("program_buffer"))
    except ITEContentFileException as error:
        print(repr(error), file=sys.stderr)

